@@ -205,7 +205,6 @@ forward (HTMLCursor *cursor)
 	return retval;
 }
 
-#if 0
 static gboolean
 forward_in_flow (HTMLCursor *cursor)
 {
@@ -230,7 +229,6 @@ forward_in_flow (HTMLCursor *cursor)
 
 	return retval;
 }
-#endif
 
 gboolean
 html_cursor_forward (HTMLCursor *cursor, HTMLEngine *engine)
@@ -272,7 +270,6 @@ backward (HTMLCursor *cursor)
 	return retval;
 }
 
-#if 0
 static gboolean
 backward_in_flow (HTMLCursor *cursor)
 {
@@ -297,7 +294,6 @@ backward_in_flow (HTMLCursor *cursor)
 
 	return retval;
 }
-#endif
 
 gboolean
 html_cursor_backward (HTMLCursor *cursor,
@@ -590,7 +586,7 @@ html_cursor_end_of_line (HTMLCursor *cursor,
 				     &x, &prev_y);
 
 	while (1) {
-		if (! forward (cursor))
+		if (! forward_in_flow (cursor))
 			return TRUE;
 
 		html_object_get_cursor_base (cursor->object, engine->painter, cursor->offset,
@@ -626,7 +622,7 @@ html_cursor_beginning_of_line (HTMLCursor *cursor,
 				     &x, &prev_y);
 
 	while (1) {
-		if (! backward (cursor))
+		if (! backward_in_flow (cursor))
 			return TRUE;
 
 		html_object_get_cursor_base (cursor->object, engine->painter, cursor->offset,
