@@ -2,6 +2,7 @@
 /*  This file is part of the GtkHTML library.
 
     Copyright (C) 1999, 2000 Helix Code, Inc.
+    Copyright (C) 2001 Ximian, Inc.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -18,7 +19,8 @@
     the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
 
-    Author: Ettore Perazzoli <ettore@helixcode.com>
+    Authors: Ettore Perazzoli <ettore@helixcode.com>
+             Radek Doulik     <rodo@ximian.com>
 */
 
 
@@ -33,6 +35,7 @@
 #include "htmlclueflow.h"
 #include "htmlcursor.h"
 #include "htmllinktext.h"
+#include "htmltable.h"
 #include "htmltext.h"
 #include "htmltextslave.h"
 #include "htmlimage.h"
@@ -40,11 +43,13 @@
 #include "htmlselection.h"
 #include "htmlundo.h"
 
+#include "htmlengine-edit.h"
 #include "htmlengine-edit-cut-and-paste.h"
 #include "htmlengine-edit-cursor.h"
 #include "htmlengine-edit-movement.h"
 #include "htmlengine-edit-selection-updater.h"
-#include "htmlengine-edit.h"
+#include "htmlengine-edit-table.h"
+#include "htmlengine-edit-tablecell.h"
 
 
 void
@@ -619,3 +624,53 @@ html_engine_break_and_fill_line (HTMLEngine *e)
 	html_engine_thaw (e);
 	html_undo_level_end (e->undo);
 }
+
+gboolean
+html_engine_next_cell (HTMLEngine *e, gboolean create)
+{
+	HTMLTableCell *cell, *current_cell;
+
+	cell = html_engine_get_table_cell (e);
+	if (cell) {
+		html_engine_hide_cursor (e);
+		do {
+			html_cursor_end_of_line (e->cursor, e);
+			html_cursor_forward (e->cursor, e);
+			current_cell = html_engine_get_table_cell (e);
+		} while (current_cell == cell);
+			
+		if (create && HTML_IS_TABLE (e->cursor->object)) {
+			html_cursor_backward (e->cursor, e);
+			html_engine_insert_table_row (e, TRUE);
+		}
+		html_engine_show_cursor (e);
+		gtk_html_edit_make_cursor_visible (e->widget);
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+gboolean
+html_engine_prev_cell (HTMLEngine *e)
+{
+	HTMLTableCell *cell, *current_cell;
+
+	cell = html_engine_get_table_cell (e);
+	if (cell) {
+		html_engine_hide_cursor (e);
+		do {
+			html_cursor_beginning_of_line (e->cursor, e);
+			html_cursor_backward (e->cursor, e);
+			current_cell = html_engine_get_table_cell (e);
+		} while (current_cell == cell);
+
+		html_engine_show_cursor (e);
+		gtk_html_edit_make_cursor_visible (e->widget);
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
